/*
 * chart [-v|-p|-l] [infile [outfile]]
 *
 * Tim Morgan
 * 8/4/89
 *
 * Rewritten by Bruce Sherrod
 *
 *
 */

#define MAXSETS         10
#define	MAXPOINTS	100
#define TWOPI          (3.141592653589793238*2.0)
#define SHADES		9    /* total number of shades */

/* #define PTPERIN	72.27 */
#define PTPERIN		1000
/* a "point" is now one thousanth of an inch.  All references to "pts"
  in this program refers to milli-inches.*/

#define BAR_THICKNESS	0.0   /* these are the default values for */
#define OFFSET 		0.0   /* the corresponding variables.     */
#define TSEP  		120.0 /* other variables are set in read_input() */
#define SEP 		0.0   /* or have "floating" defaults. */
#define RADIUS 		2.0
#define TOTALHEIGHT 	4.0  
#define TOTALWIDTH 	5.0  
#define BOTTOM		0     /* for bar labels (int label) */
#define NEAR		1

/*  Note: these lines should be added when strtok() is not available: */
#include "str.h"
#define strtok strtok2
#define toint(x) (int) (x<0?x-0.5:x+0.5)
#define abs(x) (x<0?-1*x:x)

#include "header.h"
#include "charter.h"
#include <stdio.h>
#include <ctype.h>
#include <string.h>
 
#define	Fprintf	(void) fprintf
#define	Fgets	(void) fgets
#define Sprintf (void) sprintf
#define Sscanf  (void) sscanf
#define Strcpy  (void) strcpy
extern double atof(), sin(), cos(), ceil();
extern char *index(), *malloc(), *strcpy();
 
FILE	*infile=stdin,		    /* Source of data input */
	*outfile=stdout;	    /* Destination of graph info */
float y[MAXSETS][MAXPOINTS];   	    /* Data values */
char *msg[MAXSETS][MAXPOINTS];	    /* Text which goes with data point */
double	totalwidth=TOTALWIDTH,	    /* Width of graph in inches */
	totalheight=TOTALHEIGHT,    /* Total height of graph in inches */
	bar_thickness=BAR_THICKNESS,/* thickness of bars in points */
	sep=SEP,		    /* Separation between bars in points */
	tsep=TSEP,		    /* Separation of titles from graph, pts */
	radius=RADIUS,		    /* Circle radius in inches */
	offset=OFFSET;		    /* distance between bars in different sets,
				       in milli-inches (pts) */
double 	uperin,    	            /* scaling factors for axis labeling */
	inperu,
        max,min;
int	vertical=0,		    /* Draw vertical bar chart */
        interval = -1,		    /* distance in units between tick marks. */
	npoints[MAXSETS],	    /* Number of (used) data points per set */
        shade[MAXSETS],		    /* shading pattern for each set */
	shadedensity[SHADES],       /* distance (mi) between lines a shading
				       pattern */
        label=BOTTOM,
	tnpoints=0,		    /* Total number of (used) data points */
        nsets=0,                    /* Number of (used) sets */
	pie=0;			    /* Draw a pie chart */



/* This routine simply reads the input file into the data arrays */
void read_input()
{
   char buf[200], buf2[200], *cp, *waste;
   /* unfortunately, strtok() is destructive to the poor first argument.
      Because of this, buf is copied to buf2 and then buf2 is fed to strtok
      in the data set section, since the whole buf is needed. */
   unsigned int len, i, done =0;

   
   /* initialize npoints[all values] to 0, and initialize shading patterns
      and shadedensities. Come to think of it, this probably shouldn't
      be here, but it needs to be done. */
   for (i = 0; i < MAXSETS; i++) { shade[i] = i; npoints[i] = 0; }
   shadedensity[0]=0;  /* silly      */
   shadedensity[1]=0;  /* silly      */
   shadedensity[2]=0;  /* silly      */
   shadedensity[3]=40; /* horiz      */
   shadedensity[4]=40; /* vert       */
   shadedensity[5]=50; /* cross      */
   shadedensity[6]=50; /* diag       */
   shadedensity[7]=50; /* neg_diag   */
   shadedensity[8]=60; /* cross_daig */

   Fgets(buf, sizeof(buf), infile);
   Strcpy(buf2,buf);
   waste = malloc(sizeof(buf));  /* waste can never be longer than buf */
   if (waste=strtok(buf2,"\n "),!strcmp(waste,"settings:")) { /* read settings */
#ifdef DEBUG
       Fprintf(stderr,"found settings section\n");
#endif
       while (!done) {
           Fgets(buf, sizeof(buf), infile);
           if (Strcpy(buf2,buf),waste=strtok(buf2," ="),
             !strcmp(waste,"bar_thickness")) {
               waste=strtok(NULL," =");
               bar_thickness = atof(waste);
           }
	   else if (Strcpy(buf2,buf),waste=strtok(buf2," ="),
             !strcmp(waste,"offset")) {
               waste=strtok(NULL," =");
	       offset = atof(waste);
	   }
           else if (Strcpy(buf2,buf),waste=strtok(buf2," ="),
             !strcmp(waste,"tsep")) {
               waste=strtok(NULL," =");
               tsep = atof(waste);
           }
           else if (Strcpy(buf2,buf),waste=strtok(buf2," ="),
             !strcmp(waste,"sep")) {
               waste=strtok(NULL," =");
               sep = atof(waste);
           }
           else if (Strcpy(buf2,buf),waste=strtok(buf2," ="),
             !strcmp(waste,"radius")) {
               waste=strtok(NULL," =");
               radius = atof(waste);
           }
           else if (Strcpy(buf2,buf),waste=strtok(buf2," ="),
             !strcmp(waste,"totalheight")) {
               waste=strtok(NULL," =");
               totalheight = atof(waste);
           }
           else if (Strcpy(buf2,buf),waste=strtok(buf2," ="),
             !strcmp(waste,"totalwidth")) {
               waste=strtok(NULL," =");
               totalwidth = atof(waste);
           }
           else if (Strcpy(buf2,buf),waste=strtok(buf2," ="),
             !strcmp(waste,"interval")) {
               waste=strtok(NULL," =");
               interval = atoi(waste);
           }
           else if (Strcpy(buf2,buf),waste=strtok(buf2," ="),	   
             !strcmp(waste,"shadedensity")) {
	       char *waste2="xxxxxx";
	       waste2=strtok(NULL," ");
               waste=strtok(NULL," ");
               shadedensity[atoi(waste2)] = atoi(waste);
	   }
           else if (Strcpy(buf2,buf),waste=strtok(buf2," ="),
             !strcmp(waste,"shade")) {
	       char *waste2="xxxxxx";
               waste2=strtok(NULL," ");
	       waste=strtok(NULL," ");
               shade[atoi(waste2)] = atoi(waste);
	   }
	   else if (Strcpy(buf2,buf),waste=strtok(buf2," ="),
	      !strcmp(waste,"chart")){
		waste=strtok(NULL," \n");
		/* because the default is horizontal, and the
		   switches override the settings, nothing needs to be
		   done for "chart horizontal" */
		if (!strcmp(waste,"vertical")) vertical++;
		else if (!strcmp(waste,"pie")) pie++;
	   }
	   else if (Strcpy(buf2,buf),waste=strtok(buf2," ="),
	      !strcmp(waste,"label")){
		waste=strtok(NULL," \n");
		/* because the default is bottom, nothing
		   needs to be done for "label bottom" */
		if (!strcmp(waste,"near")) label=NEAR;
	   }
           else if (Strcpy(buf2,buf),waste=strtok(buf2,"\n "),
             !strcmp(waste,"data:")) done =1;
           else Fprintf(stderr, "Setting not understood:\n    %s\n",buf);
      } /* while */
   } /* end settings section (end if) */
#ifdef DEBUG
   else Fprintf(stderr,"settings section not found.\n");
#endif
 
   while (fgets(buf, sizeof(buf), infile) != NULL) {
       Strcpy(buf2,buf);
       if (waste=strtok(buf2,"\n "),!strcmp(waste,"data:")) nsets++;  
       else { /* read a data pair from buf */
	        char *temp="xxxxxxxxxx";
		
	        Sscanf(buf, "%g", &y[nsets][npoints[nsets]]);
		Sscanf(buf, "%s", temp);
		Strcpy(waste,buf+ strlen(temp));
       	        cp = index(waste, '\n');
	        if (cp) *cp = '\0';
    	        cp = waste;
	        while (isspace(*cp)) ++cp;
	        len = strlen(cp) + 1;
	        msg[nsets][npoints[nsets]] = malloc(len); 
	        (void) Strcpy(msg[nsets][npoints[nsets]++],cp);
		tnpoints++;
       } /* end if */
   }  /* while */
#ifdef DEBUG
   Fprintf(stderr,"input read.\n");
#endif
} /* read_input */
 
 
#ifdef DEBUG
/* This routine dumps all interesting data to stderr. */   
void dump_info()
{
    int cur_set = 0, 
        cur_point = 0;
 
    Fprintf(stderr,"totalwidth=%3.2f, totalheight=%3.2f, interval=%d\n",
        totalwidth, totalheight, interval);
    Fprintf(stderr, "bar_thickness=%3.2f sep=%3.2f, tsep=%3.2f, radius=%3.2f\n", bar_thickness,sep, tsep, radius);
    Fprintf(stderr, "tnpoints=%d, max=%3.2f, min=%3.2f, label=%d\n",
	tnpoints,max,min,label);
    Fprintf(stderr, "uperin=%6.4f, inperu=%6.4f, nsets=%d\n",
	uperin,inperu,nsets);
    
#ifdef show_shades
    for (cur_set=0; cur_set<SHADES; cur_set++)
	Fprintf(stderr,"shadedensity[%d]=%d\n",cur_set,shadedensity[cur_set]);
    for (cur_set=0; cur_set<=nsets; cur_set++)
	Fprintf(stderr,"shade[%d]=%d\n",cur_set,shade[cur_set]);
#endif
    
#ifdef show_sets    
    for (cur_set = 0; cur_set <= nsets; cur_set++) {
      Fprintf(stderr, "Set %d:\n", cur_set);
      for (cur_point = 0; cur_point < npoints[cur_set]; cur_point++) {
          Fprintf(stderr,"(%3.2f,\"%s\")\n",
              y[cur_set][cur_point],msg[cur_set][cur_point]);
      }
    }
#endif
    
} /* dump_info */
#endif
 
/*******************************  MAIN *******************************/
main(argc, argv)
char *argv[];
{
    int i;

    for (i=1; i<argc; i++) if (argv[i][0] != '-') {    
	if (infile == stdin) {
	    infile = fopen(argv[i], "r");
	    if (!infile)
		perror(argv[i]),exit(1);
	} else {
	    if (outfile != stdout) {
		Fprintf(stderr, "All file arguments already specified: %s\n",
		    argv[i]);
		exit(1);
	    }
	    outfile = fopen(argv[i], "w");
	    if (!outfile)
		perror(argv[i]),exit(1);
	}
    }
    
    if (pie && vertical) {
	Fprintf(stderr, "Can't draw a pie chart and a vertical bar chart\n");
	exit(1);
    }
    
    read_input();
    
    for (i=1; i<argc; i++) if (argv[i][0] == '-') switch (argv[i][1]) {
	case 'v':
	    vertical++;
	    break;
	case 'p':
	    pie++;
	    break;
        case 'b':
	    argv[i]+=2;
	    bar_thickness = atof(argv[i]);
	    break;
	case 'l':
	    switch (argv[i][2]) {
	        case 'b':
		    label=BOTTOM;
		    break;
		case 'n':
		    label=NEAR;
		    break;
	    }
	case 'o':
	    argv[i]+=2;
	    offset = atof(argv[i]);
	    break;
	case 't':
	    argv[i]+=2;
	    tsep = atof(argv[i]);
	    break;
	case 's':
	    argv[i]+=2;
	    sep = atof(argv[i]);
	    break;
	case 'r':
	    argv[i]+=2;
	    radius = atof(argv[i]);
	    break;
	case 'H':
	    argv[i]+=2;
	    totalheight = atof(argv[i]);
	    break;
	case 'W':
	    argv[i]+=2;
	    totalwidth = atof(argv[i]);
	    break;
	case 'i':
	    argv[i]+=2;
	    interval = atoi(argv[i]);
	    break;
 
	default:
	    Fprintf(stderr, "Unknown flag %s, ignored\n", argv[i]);
    }

    picstart();
    if (vertical){
	vertical_output();
    }
    else if (pie){
	int i;
	for(i=0;i<=nsets;i++) pie_output(i);
    }
    else {
	horizontal_output();
    }
    picend();
    
    exit(0);
}
